package com.cj.jiaqingjiayi.controller;


import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConfig;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.request.AlipayUserInfoShareRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.alipay.api.response.AlipayUserInfoShareResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.contant.UserConstant;
import com.cj.jiaqingjiayi.mapper.UserMapper;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.utils.RandomNumberGenerator;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/Alipay")
public class AlipayController {


    @Value("${alipay.appId}")
    public String appId;

    @Value("${alipay.appPrivateKey}")
    public String appPrivateKey;

    @Value("${alipay.alipayPublicKey}")
    public String PublicKey;

    @Resource
    private UserMapper userMapper;

    @Resource
    private RandomNumberGenerator randomNumberGenerator;

    private static String authToken;


    /**
     * 解析code获取open_id和token
     * @return aaa
     * @throws AlipayApiException 支付宝api异常
     */
    @GetMapping("/parseCode")
    public BaseResponse<User> login(String authcode, HttpServletRequest req) throws AlipayApiException {
        String privateKey = appPrivateKey;
        String alipayPublicKey = PublicKey;
        AlipayConfig alipayConfig = new AlipayConfig();
        //沙箱网关
        alipayConfig.setServerUrl("https://openapi-sandbox.dl.alipaydev.com/gateway.do");
        //支付宝网关
        alipayConfig.setServerUrl("https://openapi.alipay.com/gateway.do");
        alipayConfig.setAppId(appId);
        alipayConfig.setPrivateKey(privateKey);
        alipayConfig.setFormat("json");
        alipayConfig.setAlipayPublicKey(alipayPublicKey);
        alipayConfig.setCharset("UTF-8");
        alipayConfig.setSignType("RSA2");
        AlipayClient alipayClient = new DefaultAlipayClient(alipayConfig);
        AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
        request.setCode(authcode);
        request.setGrantType("authorization_code");
        AlipaySystemOauthTokenResponse response = alipayClient.execute(request);
//        System.out.println(response.getBody());打印所有响应
        authToken = response.getAccessToken();
        if (!response.isSuccess()) {
            return ResultUtils.error(ErrorCode.PARAMS_ERROR);
        }
        AlipayUserInfoShareRequest request1 = new AlipayUserInfoShareRequest();
        AlipayUserInfoShareResponse response1 = alipayClient.execute(request1, authToken);
        User oid = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getOpenId, response.getOpenId()));
        if (response1.isSuccess()) {
            if (oid == null) {
                User user = new User();
                user.setOpenId(response.getOpenId());
                user.setUsername(StringUtils.isAnyBlank(response1.getNickName()) ? "随机名字" + randomNumberGenerator.generateRandomNumber() : response1.getNickName());
                user.setUserPassword("123456");
                user.setAvatarUrl(StringUtils.isAnyBlank(response1.getAvatar()) ? "https://tfs.alipayobjects.com/images/partner/ATuihpR50zu7UAAAAAAAAAAAAADtl2AA" : response1.getAvatar());
                userMapper.insert(user);
                req.getSession().setAttribute(UserConstant.USER_LOGIN_STATE,user);
                return ResultUtils.success(user,"注册成功");
            }
            req.getSession().setAttribute(UserConstant.USER_LOGIN_STATE,oid);
            return ResultUtils.success(oid,"登录成功");
        }
        req.getSession().setAttribute(UserConstant.USER_LOGIN_STATE,oid);
        return ResultUtils.success(oid,"登录成功");
    }



}