package com.cj.jiaqingjiayi.controller;

import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.*;
import com.cj.jiaqingjiayi.model.request.manicurist.ManicuristAddRequest;
import com.cj.jiaqingjiayi.model.request.manicurist.ManicuristUpdateRequest;
import com.cj.jiaqingjiayi.model.request.manicurist.MyManicuristUpdateRequest;
import com.cj.jiaqingjiayi.service.ManicuristService;
import com.cj.jiaqingjiayi.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import java.util.List;

import static com.cj.jiaqingjiayi.contant.UserConstant.USER_SALT;

@RestController
@Slf4j
@RequestMapping("/manicurist")
public class ManicuristController {

    @Resource
    private ManicuristService manicuristService;

    @Resource
    private UserService userService;


    /**
     * 添加美甲师
     * @param manicuristAddRequest
     * @return
     */
    @PostMapping("/add")
    public BaseResponse<Long> addManicurist(@RequestBody ManicuristAddRequest manicuristAddRequest,HttpServletRequest request) {
        //判断是否为管理员
        userService.isAdmin(request);

        ThrowUtils.throwIf(manicuristAddRequest == null, ErrorCode.PARAMS_ERROR);

        User user = new User();
        BeanUtils.copyProperties(manicuristAddRequest, user);
        Manicurist manicurist = new Manicurist();
        BeanUtils.copyProperties(manicuristAddRequest, manicurist);
        ManicuristAuth manicuristAuth = new ManicuristAuth();
        BeanUtils.copyProperties(manicuristAddRequest, manicuristAuth);

        //对每个应该校验的业务进行校验
        manicuristService.validUser(user, true);
        manicuristService.validManicurist(manicurist, true);
        manicuristService.validManicuristAuth(manicuristAuth, true);

        //加密
        String encryptPassword = DigestUtils.md5DigestAsHex((USER_SALT + user.getUserPassword()).getBytes());
        user.setUserPassword(encryptPassword);

        //更改用户权限为美甲师
        user.setUserRole(3);

        //往user表中补充美甲师信息
        user.setUsername(manicurist.getName());
        user.setAvatarUrl(manicurist.getManicuristAvatar());
        user.setPhone(manicurist.getPhone());

        //插入到表中
        manicuristService.addManicurist(user, manicurist, manicuristAuth);

        return ResultUtils.success(manicurist.getId(),"添加美甲师成功");
    }

    /**
     * 删除美甲师
     */
    @PostMapping("/deleteMan")
    public BaseResponse<Boolean> DeleteManicurist(long id, HttpServletRequest request){
        //判断是否为管理员
        userService.isAdmin(request);

        manicuristService.deleteMan(id);
        return ResultUtils.success(true);
    }

    /**
     * 管理员修改美甲师信息
     * 确认一定会传入美甲师的id？
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> UpdateManicurist(@RequestBody ManicuristUpdateRequest manicuristUpdateRequest, HttpServletRequest request){
        //判断是否为管理员
        userService.isAdmin(request);

        if (manicuristUpdateRequest == null || manicuristUpdateRequest.getId() == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "传入参数为空");
        }

        Manicurist manicurist = new Manicurist();

        if(manicuristUpdateRequest.getBusinessId() != null){
            //检验美甲师绑定的商家是否存在
            manicuristService.validManBus(manicuristUpdateRequest.getBusinessId());
        }

        //校验一下
        manicuristService.validManicurist(manicurist, false);

        BeanUtils.copyProperties(manicuristUpdateRequest, manicurist);

        boolean result = manicuristService.updateById(manicurist);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "修改成功");
    }

    /**
     *美甲师修改个人信息
     */
    @PostMapping("/myUpdate")
    public BaseResponse<Boolean> MyUpdateManicurist(@RequestBody MyManicuristUpdateRequest myUpdateRequest, HttpServletRequest request){
        ThrowUtils.throwIf(myUpdateRequest == null, ErrorCode.NULL_ERROR);

        User logingUser = userService.getLogingUser(request);

        Manicurist manicuristLog = manicuristService.getById(logingUser);
        Long manId = manicuristLog.getId();

        Manicurist manicurist = new Manicurist();
        BeanUtils.copyProperties(myUpdateRequest, manicurist);
        manicuristService.validManicurist(manicurist, false);
        manicurist.setId(manId);

        boolean result = manicuristService.updateById(manicurist);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "修改失败");
        return ResultUtils.success(true, "修改成功");
    }

    /**
     * 根据id查询美甲师(管理员)
     */
    @GetMapping("/queryById")
    public BaseResponse<Manicurist> QueryByIdManicurist(long manicuristId, HttpServletRequest request){
        userService.isAdmin(request);

        ThrowUtils.throwIf(manicuristId < 0, ErrorCode.NULL_ERROR, "id不符合规范");
        Manicurist manicurist = manicuristService.getById(manicuristId);
        ThrowUtils.throwIf(manicurist == null, ErrorCode.PARAMS_ERROR, "美甲师不存在");

        return ResultUtils.success(manicurist);
    }

    /**
     * 查询全部美甲师(管理员)
     */
    @PostMapping("/queryAll")
    public BaseResponse<List<Manicurist>> QueryAllManicurist(HttpServletRequest request){
        userService.isAdmin(request);

        List<Manicurist> manicuristList = manicuristService.list();
        return ResultUtils.success(manicuristList);
    }
}
