package com.cj.jiaqingjiayi.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.user.UserSearchRequest;
import com.cj.jiaqingjiayi.model.request.user.UserUpdatePasswordRequest;
import com.cj.jiaqingjiayi.model.vo.LoginUserVO;

import javax.servlet.http.HttpServletRequest;

import static com.cj.jiaqingjiayi.contant.UserConstant.ADMIN_ROLE;
import static com.cj.jiaqingjiayi.contant.UserConstant.USER_LOGIN_STATE;


public interface UserService extends IService<User> {
    /**
     * 用户注册
     *
     * @param userAccount 登陆账号
     * @param userPassword 登录密码
     * @param checkPassword 校验密码
     * @return 新用户id
     */
    long userRegister(String userAccount, String userPassword, String checkPassword);

    /**
     *
     * @param userAccount 登陆账号
     * @param userPassword 登陆密码
     * @param request 返回请求
     * @return 脱敏后的用户信息
     */
    LoginUserVO userLogin(String userAccount, String userPassword, HttpServletRequest request);

    /**
     * 用户注销
     * @param request
     * @return
     */
    int userLogout(HttpServletRequest request);

    /**
     * 用户脱敏
     * @param user
     * @return
     */
    User getSafetyUser(User user);

    /**
     * 获取当前用户登录信息
     *
     * @param request
     * @return
     */
    User getLogingUser(HttpServletRequest request);

    /**
     * 用户修改密码
     * @param userUpdatePasswordRequest
     * @param request
     * @return
     */
    boolean updateUserPassword(UserUpdatePasswordRequest userUpdatePasswordRequest, HttpServletRequest request);

    /**
     * 分页条件
     * @param searchRequest
     * @return
     */
    QueryWrapper<User> getQueryWrapper(UserSearchRequest searchRequest);

    LoginUserVO getLoginUserVO(User user);

    /**
     * 判断是否为管理员
     * @return
     */
    void isAdmin(HttpServletRequest request);
}
