package com.cj.jiaqingjiayi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Specifications;
import com.cj.jiaqingjiayi.model.domain.SpecificationsCommodities;
import com.cj.jiaqingjiayi.model.vo.SpecificationsVO;
import com.cj.jiaqingjiayi.service.AttributeService;
import com.cj.jiaqingjiayi.service.SpecificationsCommoditiesService;
import com.cj.jiaqingjiayi.mapper.SpecificationsCommoditiesMapper;
import com.cj.jiaqingjiayi.service.SpecificationsService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
* @author 高木
* @description 针对表【specifications_commodities(商品和规格的中间表)】的数据库操作Service实现
* @createDate 2024-08-21 16:21:41
*/
@Service
public class SpecificationsCommoditiesServiceImpl extends ServiceImpl<SpecificationsCommoditiesMapper, SpecificationsCommodities>
    implements SpecificationsCommoditiesService{

    @Resource
    private SpecificationsService specificationsService;

    @Resource
    private AttributeService attributeService;

    @Override
    public Boolean add(Long commoditiesId, List<Long> specificationsIds) {
        if (CollectionUtils.isEmpty(specificationsIds)) {
            return true;
        }

        for (Long specificationsId: specificationsIds) {
            SpecificationsCommodities specificationsDishes = new SpecificationsCommodities();
            specificationsDishes.setSpecificationsId(specificationsId);
            specificationsDishes.setCommoditiesId(commoditiesId);
            boolean save = this.save(specificationsDishes);
            ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        }
        return true;
    }

    @Override
    public void deleteByCommoditiesId(Long commoditiesId) {

        QueryWrapper<SpecificationsCommodities> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("commoditiesId",commoditiesId);
        long count = this.count(queryWrapper);
        if (count < 1) {
            return;
        }
        boolean remove = this.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);
    }

    @Override
    public Boolean updateByCommoditiesId(Long commoditiesId, List<Long> specificationsIds) {
        //删除之前的中间表信息
        deleteByCommoditiesId(commoditiesId);
        //保存更新的中间表信息
        return add(commoditiesId,specificationsIds);
    }

    @Override
    public List<SpecificationsVO> getSpecificationsAddAttribute(List<SpecificationsCommodities> specificationsCommoditiesList) {
        List<SpecificationsVO> specificationsVOList = specificationsCommoditiesList.stream().map(item -> {
            Long specificationsId = item.getSpecificationsId();
            Specifications specifications = specificationsService.getById(specificationsId);
            SpecificationsVO specificationsVO = new SpecificationsVO();
            BeanUtils.copyProperties(specifications, specificationsVO);
            return specificationsVO;
        }).collect(Collectors.toList());
        return attributeService.getBySpecificationsVoId(specificationsVOList);
    }


}




